/*
 * This code is for the proto v1 version of NeuroCuddl which uses the AMG8833 (GridEye) sensor for temperature sensing. 
 * This includes the control sequence for the VR game to drive the Peltier & vibration motors and electrostimulation haptics.
 * 
 * The control sequence is provided through bluetooth using the HC-05. The game outputs a string in the following format
 * "1:Pelt1,2:Pelt2,3:ELS"
 * Where Pelt1, Pelt2 are 8 bit bipolar(Forward & Reverse) PWM values (0-510) to drive the peltier elements and ELS is 12 bit (0-4095) for driving the electrostimulation stimulator
 * An example sequence would be "1:200,2:400,3:2000" would cool peltier 1 and heat peltier 2 and provide medium stimulation. It is vital to end each sequence with "\n" and not carriage return.
 * 
 */
#include <Wire.h>
#include <Adafruit_AMG88xx.h>

#define AI1 20 //any pin
#define AI2 21 //must be PWM capable pin
#define BI1 22 //any pin
#define BI2 23 //must be PWM capable pin

Adafruit_AMG88xx amg;

float pixels[AMG88xx_PIXEL_ARRAY_SIZE];

//int val1 = 1;
//char cmd[100];
//int cmdIndex;
//long timediff=0;
//long lastpeak=0;
//long currenttime;
//long lastpeltupdate = 0;
//bool boot = 1;
String xval,yval,zval;
int dir1=0,dir2=0,spd1=0,spd2=0; //direction of peltier cool/hot and intensity of stimulation
String getValue(String data, char separator, int index)
{
    int found = 0;
    int strIndex[] = { 0, -1 };
    int maxIndex = data.length() - 1;

    for (int i = 0; i <= maxIndex && found <= index; i++) {
        if (data.charAt(i) == separator || i == maxIndex) {
            found++;
            strIndex[0] = strIndex[1] + 1;
            strIndex[1] = (i == maxIndex) ? i+1 : i;
        }
    }
    return found > index ? data.substring(strIndex[0], strIndex[1]) : "";
}
void setup() {
  pinMode(AI1, OUTPUT);
  pinMode(AI2, OUTPUT);
  pinMode(BI1, OUTPUT);
  pinMode(BI2, OUTPUT);
  analogWriteResolution(12);
  pinMode(A14,OUTPUT); // Configure electrostimulation DAC
  analogWrite(A14,0);
  Serial.begin(9600);
  Serial1.begin(9600);
  amg.begin();
}

void loop() 
{

// int sensorValue = analogRead(A0);
// Serial1.println(sensorValue);
//Get 64 float array of temperature values in pixels from GridEye
//Refreshes every 100ms

amg.readPixels(pixels);
float amb_temp = amg.readThermistor();
float mean_temp = 0;
float tot_t = 0;
for(int i=1; i<=AMG88xx_PIXEL_ARRAY_SIZE; i++)
{
 mean_temp+=pixels[i-1];
}
mean_temp=mean_temp/AMG88xx_PIXEL_ARRAY_SIZE;
for(int i=1; i<=AMG88xx_PIXEL_ARRAY_SIZE; i++){
  tot_t+=pixels[i-1]-amb_temp; // Normalize each temperature value with the ambient temperature.
}
float resp_temp = tot_t/AMG88xx_PIXEL_ARRAY_SIZE;
 Serial1.println(resp_temp);
 currenttime = millis();
 // 
 String inp = Serial1.readStringUntil('\n');
 Serial.print("Inp:");
 Serial.println(inp);
 xval = getValue(inp, ':', 1);
 yval = getValue(inp, ':', 2);
 zval = getValue(inp, ':', 3);
 
// int xvalue = stringToNumber(xval);
 Serial.println("Strings");
 Serial.println(xval);
 Serial.println(yval);
 Serial.println(zval); 
 if (xval<255)
 {
  dir1=0;
  spd1=xval;
 }
 else if(xval>=255)
 {
  dir1=1;
  spd1=xval-255;
 }
 if (yval<255)
 {
  dir2=0;
  spd2=yval;
 }
 else if(yval>=255)
 {
  dir2=0;
  spd2=yval-255;
 }
 M1(dir1,spd1);
 M2(dir2,spd2);
 
  analogWrite(A14,zval.toInt());
}

void M1(boolean dir1, int spd1)
{
analogWrite(AI2, spd1);
digitalWrite(AI1, dir1);
}

void M2(boolean dir2, int spd2)
{
analogWrite(BI2, spd2);
digitalWrite(BI1, dir2);
}

